
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "ad_skipper_43.h"
//-----------------------------------------------------------------------------

//abandon any half built cutfiles
//free the section
//create a new one
//load a cutfile if available (won't be if timeshifting)
//else start building one
int on_playback_started()
{
    if (debug & 2) TAP_Print("\nPlayback Started");
    if (pb_scan.state == STATE_IDLE) {
        pb_scan.state = STATE_OPEN;
    }
    return 0;
}

//abandon any half built cutfiles
//free the section
int on_playback_stopped()
{
    if (debug & 2) TAP_Print("\nPlayback Stopped");
    if (pb_scan.state != STATE_IDLE) {
        pb_scan.abort = 1;
        pb_scan.state = STATE_CLOSE;
    }
    return 0;
}


int on_timeshift_playback_started()
{
    if (debug & 2) TAP_Print("\nTimeshift Playback Started");
    return 0;
}

int on_timeshift_playback_stopped()
{
    if (debug & 2) TAP_Print("\nTimeshift Playback Stopped");
    return 0;
}


//handle this
//also want to continue scanning, since the user may be watching
int on_recording_playback_started()
{
    if (debug & 2) TAP_Print("\nRecording Playback Started");
//    if (pb_scan.state == STATE_IDLE) {
//        pb_scan.state = STATE_OPEN;
//    }
    return 0;
}

int on_recording_playback_stopped()
{
    if (debug & 2) TAP_Print("\nRecording Playback Stopped");
//    if (pb_scan.state != STATE_IDLE) {
//        pb_scan.abort = 1;
//        pb_scan.state = STATE_CLOSE;
//    }
    return 0;
}


//or anything else that causes the timeshift buffer to be lost
//we don't need to save this one to disk
//abandon any half built cutfiles
//free the section
int on_channel_change()  // == timeshift killed
{
    if (debug & 2) TAP_Print("\nChannel Change");
    if (pb_scan.state != STATE_IDLE) {
        pb_scan.abort = 1;
        pb_scan.state = STATE_CLOSE;
    }
    return 0;
}

//do we have a complete or partial cutfile available
//else do nothing
//if from 4:3 to 16:9
int on_playback_aspect_change()
{
    if (debug & 2) TAP_Print("\nAspect Ratio Change");
    //if from 4:3 to 16:9
    if (automatic_skip_enabled) {
        if (fw_aspect[CHANNEL_Main].ratio==ASPECT_16_9) {
            skip_to_next(ASPECT_4_3, 0);
        }
    }

    return 0;
}


//could scan this, but probably best not if mutiple recordings/playback also happening
//be nice to be alerted once the recoding finished to add it to the front of the queue to be scanned
//maybe scan live with low priority
int on_recording_but_not_playing()
{
    if (debug & 2) TAP_Print("\non_recording_but_not_playing()");
    return 0;
}

//-----------------------------------------------------------------------------
//
//print multiline message to OSD
//  500=vertical position(0-576)
//  RGB(3,5,10)=background colour(dark blue)
//  200=hold time(2 seconds)
//  0=blocking
//
void oprintf(const void *fmt, ...)
{
    va_list argp;

    va_start(argp, fmt);
    vsprintf(strLog, fmt, argp);
//    TAP_Print("\n%s", strLog);
    ShowMessage_y(strLog, 500, RGB(3,5,10), 200, 0);
    va_end(argp);

    return;
}

void osd_status(type_section *section, type_scan *scan)
{
    int percent;
    int pos, len;
    char percent_str[0x10];

    if (debug & 1) {
        pos = scan->fpos64>>16;
        len = scan->flen64>>16;
        percent = (pos*100)/(len+1);
        if (percent) sprintf(percent_str," %d%%", percent);
        else sprintf(percent_str,"");

        oprintf("Scanning: %s%s%s",
            section->info.filename,
            ((scan->flags & 3) == 1) ? " [4:3]" :
            ((scan->flags & 3) == 2) ? " [16:9]" :
            ((scan->flags & 3) == 3) ? " [4:3/16:9]" : "",
            percent_str
        );
    }
}

void two_second_update()
{
    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];
    static int last_playMode = PLAYMODE_None;
    static int last_trickMode = TRICKMODE_Normal;
    int tvRadio;
    int chNum;

    //osd
    if (pb_scan.state == STATE_UPDATE) {
        osd_status(&section_playing, &pb_scan);
    }

    //playmode changed
    TAP_Hdd_GetPlayInfo(&pi);
    if (pi.playMode != last_playMode) {
        if (debug & 2) TAP_Print("\n%s to %s", get_str(playmode_str, last_playMode), get_str(playmode_str, pi.playMode));
        switch(last_playMode) {
            case PLAYMODE_None:
                break;
            case PLAYMODE_Playing:
                on_playback_stopped();
                break;
            case PLAYMODE_TempPlaying:
                on_timeshift_playback_stopped();
                break;
            case PLAYMODE_RecPlaying:
                on_recording_playback_stopped();
                break;
        }
        switch(pi.playMode) {
            case PLAYMODE_None:
                break;
            case PLAYMODE_Playing:
                on_playback_started();
                break;
            case PLAYMODE_TempPlaying:
                on_timeshift_playback_started();
                break;
            case PLAYMODE_RecPlaying:
                on_recording_playback_started();
                break;
        }

        last_playMode = pi.playMode;
    }

    //was it paused?
    if (pi.trickMode != last_trickMode) {
        if (pi.trickMode != TRICKMODE_Pause) {
            //if from 4:3 to 16:9
            if (automatic_skip_enabled) {
                if (fw_aspect[CHANNEL_Main].ratio==ASPECT_16_9) {
                    skip_to_next(ASPECT_4_3, 0);
                }
            }
        }
        last_trickMode = pi.trickMode;
    }


    //channel changed
    TAP_Channel_GetCurrent(&tvRadio, &chNum);
    if (!((tvRadio == last_tvRadio) && (chNum == last_chNum))) {
        if (debug & 2) TAP_Print("\n%s:%d to %s:%d", (last_tvRadio==SVC_TYPE_Tv)?"tv":"radio", last_chNum, (tvRadio==SVC_TYPE_Tv)?"tv":"radio", chNum);
        on_channel_change();  // == timeshift killed
        last_tvRadio = tvRadio;
        last_chNum = chNum;
    }
}

//-----------------------------------------------------------------------------
int skip_to_next(int ratio, int back)
{
    //are we playing a file?
    //do we have a cut section?
    //get the current position
    //find where we are
    //are we in 4:3 - yes, do nothing
    //are we in 16:9 - yes, skip to next 4:3

    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];
    char cut_filename[0x100];
    dword block;
    int ret;
    int i;
    dword now, next;
    int len;
    ulong64 pos64;

    if (debug & 2) TAP_Print("\nskip_to_next(ratio=%x, back=%x)", ratio, back);
    TAP_Hdd_GetPlayInfo(&pi);
    if ((pi.playMode == PLAYMODE_Playing) || (pi.playMode == PLAYMODE_TempPlaying) || (pi.playMode == PLAYMODE_RecPlaying)) {
    }
    else {
        sprintf(strLog, "Not Playing File");
        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
        return -1;
    }


    //is it paused?
    //?????
    if (pi.trickMode == TRICKMODE_Pause) {
        sprintf(strLog, "Paused File");
//        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
        log_message(LOG_SERIAL, strLog, DELAY_SHORT);
        return -1;
    }

    //is section_playing loaded?
    if (section_playing.info.loaded && (

        //is section_playing still scanning? e.g. PLAYMODE_TempPlaying/PLAYMODE_RecPlaying
        section_playing.info.scanning ||

        //is section_playing scanned?
        section_playing.info.scanned)) {
    }
    else {
        //open the cutfile
        len = strlen(pi.file->name);
        if (strncmp(&pi.file->name[len-4], ".rec", 4) != 0) {
            //if not playing a .rec file, quit
            sprintf(strLog, "Not Playing a .rec File");
            log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
            return -1;
        }

        //create .cut filename
        memcpy(&cut_filename[0], pi.file->name, len-4);
        memcpy(&cut_filename[len-4], ".cut\0", 5);
//        TAP_Print("\npi.file->name = %s, len=%x", pi.file->name, len);
//        TAP_Print("\ncut_filename = %s", cut_filename);

        //load the cutfile
        ret = load_cutfile(cut_filename, &section_playing);
        if (ret < 0) {
            sprintf(strLog, ".cut File Not Found");
            log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
            return -1;
        }
        if (ret == 1) {
            //.cut file does not exist
        }
    }
    if (debug & 2) dump_section(&section_playing);


    //is section valid?
    //is section_playing still scanning? e.g. PLAYMODE_TempPlaying/PLAYMODE_RecPlaying
    //is section_playing scanned?
    if ((section_playing.info.scanning || section_playing.info.scanned)) {
    }
    else {
        sprintf(strLog, "Section not valid");
        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
        return -1;
    }

    if (section_playing.aspect.count <= 1) {
        sprintf(strLog, "No cutpoints");
        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
        return -1;
    }

    if (pi.totalBlock == 0) {
        sprintf(strLog, "oops: pi.totalBlock = %x", pi.totalBlock);
        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
        return -1;
    }

    pos64 = BYTESPERSECTOR*188*pi.currentBlock;
    if (debug & 2) TAP_Print("\nblock: %x/%x, fpos64=%x%08x", pi.currentBlock, pi.totalBlock, *(dword *)(0+(int)&pos64), *(dword *)(4+(int)&pos64));

    for (i=0; i<(int)section_playing.aspect.count; i++) {
        block = section_playing.aspect.cut[i].fpos64 / BYTESPERSECTOR;
        block = block / 188;
        if (debug & 2) TAP_Print("\n%d: fpos64=%x%08x, block=%x, %s to %s", i,
            *(dword *)(0+(int)&section_playing.aspect.cut[i].fpos64),
            *(dword *)(4+(int)&section_playing.aspect.cut[i].fpos64),
            block,
            (section_playing.aspect.cut[i].from==ASPECT_4_3) ? "4:3" : (section_playing.aspect.cut[i].from==ASPECT_16_9) ? "16:9" : "??:?",
            (section_playing.aspect.cut[i].to  ==ASPECT_4_3) ? "4:3" : (section_playing.aspect.cut[i].to  ==ASPECT_16_9) ? "16:9" : "??:?"
        );
    }

    //find where we are
    //are we in 4:3 - yes, leave
    //are we in 16:9 - yes, skip to next 4:3

    if (back) {
        for (i=0; i<(int)section_playing.aspect.count; i++) {
            now = section_playing.aspect.cut[i].from;
            next = section_playing.aspect.cut[i].to;
            block = section_playing.aspect.cut[i].fpos64 / BYTESPERSECTOR;
            block = block / 188;

            //scan cutlist until just past current position
            //then go back 1
            if ((block > pi.currentBlock) && (i>0)) {
                now = section_playing.aspect.cut[i-1].from;
                next = section_playing.aspect.cut[i-1].to;
                block = section_playing.aspect.cut[i-1].fpos64 / BYTESPERSECTOR;
                block = block / 188;
                if ((now == ASPECT_16_9) && (next == ASPECT_4_3)) {
                    if (ratio == ASPECT_4_3) {
                        sprintf(strLog, "Going Back to End of Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                        if (debug & 2) TAP_Print("\nTAP_Hdd_ChangePlaybackPos(%x)", block);
//                        ret = TAP_Hdd_ChangePlaybackPos(block);
                        ret = exTAP_CallFirmware(fw_fChangePlaybackPos, block, 0,0,0,0,0,0,0,0,0,0);
//                        TAP_Print(", ret=%x", ret);
                    }
                    else {
                        sprintf(strLog, "Not In Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    }
                }
                else if ((now == ASPECT_4_3) && (next == ASPECT_16_9)) {
                    if (ratio == ASPECT_16_9) {
                        sprintf(strLog, "Going Back to Start of Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                        if (debug & 2) TAP_Print("\nTAP_Hdd_ChangePlaybackPos(%x)", block);
//                        ret = TAP_Hdd_ChangePlaybackPos(block);
                        ret = exTAP_CallFirmware(fw_fChangePlaybackPos, block, 0,0,0,0,0,0,0,0,0,0);
//                        TAP_Print(", ret=%x", ret);
                    }
                    else {
                        sprintf(strLog, "In Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    }
                } else {
                    sprintf(strLog, "Not 4:3 program, or no Adverts");
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    if (debug & 2) TAP_Print("\nnow=%s, next=%s",
                        (now ==ASPECT_4_3) ? "4:3" : (now ==ASPECT_16_9) ? "16:9" : "??:?",
                        (next==ASPECT_4_3) ? "4:3" : (next==ASPECT_16_9) ? "16:9" : "??:?"
                    );
                }
                return 0;
            }
       }
    }
    else {
        for (i=0; i<(int)section_playing.aspect.count; i++) {
            now = section_playing.aspect.cut[i].from;
            next = section_playing.aspect.cut[i].to;
            block = section_playing.aspect.cut[i].fpos64 / BYTESPERSECTOR;
            block = block / 188;

            //scan cutlist until just past current position
            if (block > pi.currentBlock) {
                if ((now == ASPECT_16_9) && (next == ASPECT_4_3)) {
                    if (ratio == ASPECT_4_3) {
                        sprintf(strLog, "Going to End of Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);

                        if (debug & 2) TAP_Print("\nTAP_Hdd_ChangePlaybackPos(%x)", block);
//                        ret = TAP_Hdd_ChangePlaybackPos(block);
                        ret = exTAP_CallFirmware(fw_fChangePlaybackPos, block, 0,0,0,0,0,0,0,0,0,0);
//                        TAP_Print(", ret=%x", ret);
                    }
                    else {
                        sprintf(strLog, "In Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    }
                }
                else if ((now == ASPECT_4_3) && (next == ASPECT_16_9)) {
                    if (ratio == ASPECT_16_9) {
                        sprintf(strLog, "Going to Start of Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                        if (debug & 2) TAP_Print("\nTAP_Hdd_ChangePlaybackPos(%x)", block);
//                        ret = TAP_Hdd_ChangePlaybackPos(block);
                        ret = exTAP_CallFirmware(fw_fChangePlaybackPos, block, 0,0,0,0,0,0,0,0,0,0);
//                        TAP_Print(", ret=%x", ret);
                    }
                    else {
                        sprintf(strLog, "Not In Adverts");
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    }
                } else {
                    sprintf(strLog, "Not 4:3 program, or no Adverts");
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    if (debug & 2) TAP_Print("\nnow=%s, next=%s",
                        (now ==ASPECT_4_3) ? "4:3" : (now ==ASPECT_16_9) ? "16:9" : "??:?",
                        (next==ASPECT_4_3) ? "4:3" : (next==ASPECT_16_9) ? "16:9" : "??:?"
                    );
                }
                return 0;
            }
        }
    }
    sprintf(strLog, "No more cutpoints");
    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
    return 0;
}

//-----------------------------------------------------------------------------
